//
//  NormEditController.m
//  MeykiEFarm
//
//  Created by meyki on 1/3/17.
//  Copyright © 2017 邓龙. All rights reserved.
//

#import "NormEditController.h"
#import "ImageCollectCell.h"
#import "NormalCollectCell.h"
#import "TitleCollectCell.h"
#import "NormCollectCell.h"

#import "NormValueModel.h"
#import "AttrValueModel.h"
#import "SpecModel.h"

/** UICollectionViewCell 纵向和横向的间距 */
static NSInteger const MinimumInteritemSpacing    = 0;
/** 底部按钮高度 */
static NSInteger const BottomConfirmHeight        = 56;
/** collectionView头部及底部高度 */
static NSInteger const CollectionViewHeaderHeight = 52;
/** collectionView底部高度 */
static NSInteger const CollectionViewFooterHeight = 30;
/** 第一个区域默认item个数 */
static NSInteger const FirstSectionCellNum        = 3;
/** 其他区域默认item个数 */
static NSInteger const DefSectionCellNum          = 9;

static NormEditController *sInstance = nil;

static dispatch_once_t onceToken;

@interface NormEditController ()<UICollectionViewDelegate, UICollectionViewDataSource, UITextFieldDelegate>
{
    UICollectionView *normCollectView;
    UICollectionViewFlowLayout *flowLayout;
    NSArray *overallList;                     //获取的规格model数组
    NSMutableArray *arrayList;                //规格数组
    NSMutableArray *allList;                  //选择规格标签组合数据model数组
    NSMutableDictionary *dicList;             //金额库存字典
    NSMutableArray *space;                    //规格标签组合数组
    NormValueModel *normModel;                
    AttrValueModel *attrModel;
    UIView *alearCovewView;  //提示框蒙层
    UIView *backTView;  //提示框背景
}
@end

@implementation NormEditController

+ (NormEditController *)sharedManager
{
    static NormEditController *sharedAccountManagerInstance = nil;
    dispatch_once(&onceToken, ^{
        sharedAccountManagerInstance = [[self alloc] init];
    });
    
    return sharedAccountManagerInstance;
}

#pragma mark - 销毁单例
+ (void)destroyShareManager{
    sInstance=nil;
    onceToken=0l;
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    [self.navigationController setNavigationBarHidden:NO animated:YES];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    self.view.backgroundColor = [UIColor whiteColor];
    self.title = @"商品规格";
    
    //增加监听，当键盘出现或改变时收出消息
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(keyboardWillShow:)
                                                 name:UIKeyboardWillShowNotification
                                               object:nil];
    
    //增加监听，当键退出时收出消息
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(keyboardWillHide:)
                                                 name:UIKeyboardWillHideNotification
                                               object:nil];
    
    UITapGestureRecognizer *tap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(hiddenKeyboardAction)];
    [self.view addGestureRecognizer:tap];
    
    [self createHomeCollectionView];
    [self requestNormDataAction];
    
}

#pragma mark - 商品规格数据请求
- (void)requestNormDataAction {
    
    NSString *jsonPath = [[NSBundle mainBundle] pathForResource:@"info" ofType:@"json"];
    NSData *data=[NSData dataWithContentsOfFile:jsonPath];
    NSError *error;
    NSDictionary *params = [NSJSONSerialization JSONObjectWithData:data
                                                  options:NSJSONReadingAllowFragments
                                                    error:&error];
    
    NSArray *response = params[@"categoryAttr"];
    
    overallList = [NormValueModel mj_objectArrayWithKeyValuesArray:response];
    
    for (NormValueModel *valueModel in overallList) {
        AttrValueModel *model = [[AttrValueModel alloc] init];
        model.attrValueName = @"";
        model.attrValueId = @"";
        model.isNew = YES;
        model.isSelect = NO;
        [valueModel.attrValueList insertObject:model atIndex:0];
    }
    
    dicList = [NSMutableDictionary dictionary];
    allList = [NSMutableArray array];
    arrayList = [NSMutableArray array];
    for (NSInteger i=0; i<overallList.count; i++) {
        NSMutableArray *list = [NSMutableArray array];
        [arrayList addObject:list];
    }
    
    [normCollectView reloadData];
}

#pragma mark - 创建CollectionView
- (void)createHomeCollectionView
{
    flowLayout=[[UICollectionViewFlowLayout alloc] init];
    [flowLayout setScrollDirection:UICollectionViewScrollDirectionVertical];
    flowLayout.headerReferenceSize=CGSizeMake(viewWidth, CollectionViewHeaderHeight);
    flowLayout.footerReferenceSize=CGSizeMake(viewWidth, CollectionViewFooterHeight);

    normCollectView = [[UICollectionView alloc] initWithFrame:CGRectZero collectionViewLayout:flowLayout];
    normCollectView.backgroundColor =  [UIColor whiteColor];;
    normCollectView.dataSource=self;
    normCollectView.delegate=self; 
    [self.view addSubview:normCollectView];
    
    UIButton *confirmBtn = [[UIButton alloc] init];
    confirmBtn.backgroundColor = [UIColor_HEX colorWithHexString:@"00833f"];
    confirmBtn.titleLabel.font = [UIFont systemFontOfSize:14];
    [confirmBtn setTitleColor:[UIColor_HEX colorWithHexString:@"ffffff"] forState:UIControlStateNormal];
    [confirmBtn setTitle:@"确定" forState:UIControlStateNormal];
    confirmBtn.layer.cornerRadius = 4;
    [self.view addSubview:confirmBtn];
    [confirmBtn addTarget:self action:@selector(confirmButtonAction) forControlEvents:UIControlEventTouchUpInside];

    UIEdgeInsets edge = UIEdgeInsetsMake(0, 0, BottomConfirmHeight, 0);

    [normCollectView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.edges.equalTo(self.view).insets(edge);
    }];
    
    [confirmBtn mas_makeConstraints:^(MASConstraintMaker *make) {

        make.left.equalTo(self.view).with.offset(12);
        make.right.equalTo(self.view).with.offset(-12);
        make.bottom.equalTo(self.view).with.offset(-12);
        make.height.equalTo(@44);

    }];
    
    //注册Cell
    [normCollectView registerNib:[UINib nibWithNibName:@"ImageCollectCell" bundle:nil] forCellWithReuseIdentifier:@"ImageCollectCell"];
    
    [normCollectView registerNib:[UINib nibWithNibName:@"NormalCollectCell" bundle:nil] forCellWithReuseIdentifier:@"NormalCollectCell"];
    
    [normCollectView registerNib:[UINib nibWithNibName:@"TitleCollectCell" bundle:nil] forCellWithReuseIdentifier:@"TitleCollectCell"];
    
    [normCollectView registerNib:[UINib nibWithNibName:@"NormCollectCell" bundle:nil] forCellWithReuseIdentifier:@"NormCollectCell"];
    
    [normCollectView registerNib:[UINib nibWithNibName:@"TitleCollectCell" bundle:nil] forSupplementaryViewOfKind:UICollectionElementKindSectionHeader withReuseIdentifier:@"TitleCllectCellHeader"];
    
    [normCollectView registerNib:[UINib nibWithNibName:@"TitleCollectCell" bundle:nil] forSupplementaryViewOfKind:UICollectionElementKindSectionFooter withReuseIdentifier:@"TitleCllectCellFooter"];
}

#pragma mark -- UICollectionViewDataSource
//定义每个section的cell个数
-(NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section
{
    if (overallList.count == section) {
        return space.count;
    }
    
    normModel = overallList[section];
    if ([normModel.isShow integerValue] == 1) {
        return normModel.attrValueList.count;
    }
    else
    {
        if (section == 0 && normModel.attrValueList.count > FirstSectionCellNum && [normModel.isShow integerValue] == 0) {
            return FirstSectionCellNum;
        }
        
        if (section != 0 && normModel.attrValueList.count > DefSectionCellNum && [normModel.isShow integerValue] == 0) {
            return DefSectionCellNum;
        }

        return normModel.attrValueList.count;
    }
}

#pragma mark - 定义section数量
-(NSInteger)numberOfSectionsInCollectionView:(UICollectionView *)collectionView
{
    return overallList.count + 1;
}

-(UICollectionReusableView *)collectionView:(UICollectionView *)collectionView viewForSupplementaryElementOfKind:(NSString *)kind atIndexPath:(NSIndexPath *)indexPath{
    
    TitleCollectCell *reusableView;
    
    if (kind==UICollectionElementKindSectionHeader) {
        reusableView=[collectionView dequeueReusableSupplementaryViewOfKind:UICollectionElementKindSectionHeader withReuseIdentifier:@"TitleCllectCellHeader" forIndexPath:indexPath];
        if (indexPath.section == 0) {
            reusableView.LView.backgroundColor = [UIColor whiteColor];
        }else{
            reusableView.LView.backgroundColor = [UIColor_HEX colorWithHexString:@"efefef"];
        }
        if (indexPath.section == overallList.count) {
            [reusableView setTitleLabelAction:@"价格/库存"];
        }
        else {
            normModel = overallList[indexPath.section];
            [reusableView setTitleLabelAction:normModel.attrName];
        }
    }
    else {
        reusableView=[collectionView dequeueReusableSupplementaryViewOfKind:UICollectionElementKindSectionFooter withReuseIdentifier:@"TitleCllectCellFooter" forIndexPath:indexPath];
        reusableView.unfoldButton.tag = indexPath.section;
        [reusableView.unfoldButton addTarget:self action:@selector(unfoldButtonAction:) forControlEvents:UIControlEventTouchUpInside];
        [reusableView showUnfoldBtnAction];
        if (indexPath.section != overallList.count) {
            normModel = overallList[indexPath.section];
            reusableView.unfoldButton.selected = [normModel.isShow integerValue];
        }
        
        if (indexPath.section == 0 && normModel.attrValueList.count <= FirstSectionCellNum) {
            reusableView.unfoldButton.hidden = YES;
            return reusableView;
        }
        else if (indexPath.section != 0 && normModel.attrValueList.count <= DefSectionCellNum) {
            reusableView.unfoldButton.hidden = YES;
            return reusableView;
        }
        else if (indexPath.section == overallList.count) {
            reusableView.frame = CGRectMake(0, 0, 0, 0);
            return reusableView;
        }
    }

    return reusableView;
}

#pragma mark --UICollectionViewDelegateFlowLayout
//定义每个UICollectionView 纵向的间距
- (CGFloat)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout*)collectionViewLayout minimumInteritemSpacingForSectionAtIndex:(NSInteger)section
{
    return MinimumInteritemSpacing;
}

#pragma mark - 定义每个Item 的大小
- (CGSize)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout*)collectionViewLayout sizeForItemAtIndexPath:(NSIndexPath *)indexPath
{
    if (indexPath.section == 0) {
        return CGSizeMake(viewWidth/3, viewWidth/3 + 20);
    }
    else if (indexPath.section == overallList.count) {
        
        NSArray *arrList = space[indexPath.row];
        NSString *str = [self getStringAction:arrList];
        CGSize size = [str sizeWithAttributes:@{NSFontAttributeName: [UIFont systemFontOfSize:14]}];
        return CGSizeMake(viewWidth, 70+size.height);
    }
    else {
        return CGSizeMake(viewWidth/3, 40);
    }
}

#pragma mark - 定义每个UICollectionView 的 margin
-(UIEdgeInsets)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout *)collectionViewLayout insetForSectionAtIndex:(NSInteger)section
{
    return UIEdgeInsetsMake(0, 0, MinimumInteritemSpacing, 0 );
}

#pragma mark - 每个UICollectionView展示的内容
-(UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath
{
    if (indexPath.section != overallList.count) {
        normModel = overallList[indexPath.section];
        attrModel = normModel.attrValueList[indexPath.row];
    }
    
    if (indexPath.section == 0) {
        ImageCollectCell *cell = [collectionView dequeueReusableCellWithReuseIdentifier:@"ImageCollectCell" forIndexPath:indexPath];
        
        normModel = overallList[indexPath.section];
        [cell setPlaceholderAction:normModel.attrName];
        
        NormEditController *weakSelf = self;
        [cell refreshDataAction:^{
            [weakSelf calculateAllListCount];
        }];
        if (indexPath.row == 0) {
            cell.imgView.userInteractionEnabled = NO;
        }else{
            cell.imgView.userInteractionEnabled = YES;
        }
        if (indexPath.row == 0) {
            [cell setImageCollectCellData:attrModel firstRow:YES vc:self];
        }
        else
        {
            [cell setImageCollectCellData:attrModel firstRow:NO vc:self];
        }
        
        cell.editFiled.tag = indexPath.row;
        cell.editFiled.superview.tag = indexPath.section;
        cell.editFiled.delegate = self;
        
        cell.deleteButton.tag = indexPath.row;
        cell.deleteButton.superview.tag = indexPath.section;
        [cell.deleteButton addTarget:self action:@selector(deleteButtonAction:) forControlEvents:UIControlEventTouchUpInside];
        
        cell.topButton.tag = indexPath.row;
        cell.topButton.superview.tag = indexPath.section;
        [cell.topButton addTarget:self action:@selector(choostButtonAction:) forControlEvents:UIControlEventTouchUpInside];
        
        return cell;
    }
    else if (indexPath.section == overallList.count) {
        NormCollectCell *cell = [collectionView dequeueReusableCellWithReuseIdentifier:@"NormCollectCell" forIndexPath:indexPath];
        [cell setNameLabelAction:space[indexPath.row]];
        
        cell.sumLab.superview.tag = indexPath.row;
        cell.priceLab.superview.tag = indexPath.row;
        [cell.priceLab addTarget:self action:@selector(textFieldChangeAction:) forControlEvents:UIControlEventEditingChanged];
        [cell.priceLab addTarget:self action:@selector(textFieldDidEndAction:) forControlEvents:UIControlEventEditingDidEnd];
        [cell.sumLab addTarget:self action:@selector(textFieldChangeAction:) forControlEvents:UIControlEventEditingChanged];
        
        return cell;
    }
    else
    {
        NormalCollectCell *cell = [collectionView dequeueReusableCellWithReuseIdentifier:@"NormalCollectCell" forIndexPath:indexPath];
        
        normModel = overallList[indexPath.section];
        [cell setPlaceholderAction:normModel.attrName];
        
        if (indexPath.row == 0) {
            [cell setNormCollectCellData:attrModel firstRow:YES];
        }
        else
        {
            [cell setNormCollectCellData:attrModel firstRow:NO];
        }
        
        cell.normailText.tag = indexPath.row;
        cell.normailText.superview.tag = indexPath.section;
        cell.normailText.delegate = self;
        
        cell.normalDelete.tag = indexPath.row;
        cell.normalDelete.superview.tag = indexPath.section;
        [cell.normalDelete addTarget:self action:@selector(deleteButtonAction:) forControlEvents:UIControlEventTouchUpInside];
        
        cell.selectBtn.tag = indexPath.row;
        cell.selectBtn.superview.tag = indexPath.section;
        [cell.selectBtn addTarget:self action:@selector(choostButtonAction:) forControlEvents:UIControlEventTouchUpInside];
        
        return cell;
    }
}

#pragma mark - 隐藏键盘
- (void)hiddenKeyboardAction {
    [self.view endEditing:YES];
}

#pragma mark - textFieldDelegate
- (void)textFieldDidBeginEditing:(UITextField *)textField {
    
    if (textField.tag == 0) {
        textField.text = @"";
    }
    else
    {
        normModel = overallList[textField.superview.tag];
        attrModel = normModel.attrValueList[textField.tag];
        textField.text = attrModel.attrValueName;
    }
}

- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string {

    if (textField.tag == 0) {
        [textField resignFirstResponder];
        normModel = overallList[textField.superview.tag];
        AttrValueModel *model = [[AttrValueModel alloc] init];
        if (string.length > 10) {
            string = [string substringToIndex:10];
        }
        model.attrValueName = string;
        model.isNew = YES;
        model.isSelect = YES;
        
        NSMutableArray *list = arrayList[textField.superview.tag];
        [list addObject:model];
        
        [normModel.attrValueList insertObject:model atIndex:1];
        
        [self calculateAllListCount];
    }
    
    return YES;
}

- (void)textFieldDidEndEditing:(UITextField *)textField {
    
    normModel = overallList[textField.superview.tag];
    if (normModel.attrValueList.count > textField.tag) {
        attrModel = normModel.attrValueList[textField.tag];
        attrModel.attrValueName = textField.text;
        attrModel.isNew = YES;
        attrModel.isSelect = YES;
    }
    
    [normCollectView reloadData];
}

#pragma mark - 价格库存textField监听
- (void)textFieldChangeAction:(UITextField *)textField {
    
    NSArray *arrList = space[textField.superview.tag];
    NSString *str = [self getStringAction:arrList];
    
    NSMutableDictionary *dic = dicList[str];
    if (![DataCheck isValidDictionary:dic]) {
        dic = [NSMutableDictionary dictionary];
    }

    switch (textField.tag) {
        case 1:
            
            [dic setObject:textField.text?textField.text:@"" forKey:@"price"];
            break;
            
        case 2:
            
            [dic setObject:textField.text?textField.text:@"" forKey:@"count"];
            break;
        default:
            break;
    }
    
    [dicList setObject:dic forKey:str];
}

- (void)textFieldDidEndAction:(UITextField *)textField {
    if (textField.tag == 1) {
        textField.text = [NSString stringWithFormat:@"%.2f", [textField.text floatValue]];
    }
}

//当键盘出现或改变时调用
- (void)keyboardWillShow:(NSNotification *)aNotification
{
    //获取键盘的高度
    NSDictionary *userInfo = [aNotification userInfo];
    NSValue *aValue = [userInfo objectForKey:UIKeyboardFrameEndUserInfoKey];
    CGRect keyboardRect = [aValue CGRectValue];
    int height = keyboardRect.size.height;
    
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.1 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        UIEdgeInsets edge = UIEdgeInsetsMake(0, 0, height, 0);
        [normCollectView mas_updateConstraints:^(MASConstraintMaker *make) {
            make.edges.equalTo(self.view).insets(edge);
        }];
    });
}

//当键退出时调用
- (void)keyboardWillHide:(NSNotification *)aNotification{
    UIEdgeInsets edge = UIEdgeInsetsMake(0, 0, BottomConfirmHeight, 0);
    [normCollectView mas_updateConstraints:^(MASConstraintMaker *make) {
        make.edges.equalTo(self.view).insets(edge);
    }];
}

#pragma mark - 删除按钮点击事件
- (void)deleteButtonAction:(UIButton *)sender {
    
    normModel = overallList[sender.superview.tag];
    attrModel = normModel.attrValueList[sender.tag];

    NSMutableArray *list = arrayList[sender.superview.tag];
    [list removeObject:attrModel];
    [normModel.attrValueList removeObjectAtIndex:sender.tag];
    [self calculateAllListCount];
}

#pragma mark - 选择按钮事件
- (void)choostButtonAction:(UIButton *)sender {
    
    normModel = overallList[sender.superview.tag];
    attrModel = normModel.attrValueList[sender.tag];
    attrModel.isSelect = !attrModel.isSelect;
    
    NSMutableArray *list = arrayList[sender.superview.tag];
    if (attrModel.isSelect) {
        [list addObject:attrModel];
    }
    else
    {
        [list removeObject:attrModel];
    }
    
    [self calculateAllListCount];
}

#pragma mark - 计算选中count
- (void)calculateAllListCount {
    space = [NSMutableArray array];

    for (NSArray *list in arrayList) {
        if (list.count  == 0) {
            [normCollectView reloadData];
            return;
        }
    }
   [self descartes:arrayList result:space layer:0 curList:[NSMutableArray array]];

    [allList removeAllObjects];
    for (NSInteger i=0;i<space.count;i++) {
        NSArray *arrList = space[i];
        NSString *str = [self getStringAction:arrList];
        NSString *imgUrl = nil;
        for (AttrValueModel *attm in arrList) {
            if ([DataCheck isValidString:attm.imageUrl]) {
                imgUrl = attm.imageUrl;
            }
        }
        
        NSArray *dic = [AttrValueModel mj_keyValuesArrayWithObjectArray:arrList];
        SpecModel *model = [[SpecModel alloc] init];
        model.sku = [dic mutableCopy];
        model.specImg = imgUrl;
        
        if (overallList.count > i) {
            NormValueModel *vModel = overallList[i];
            for (NSMutableDictionary *dic in model.sku) {
                [dic setObject:vModel.attrId forKey:@"attrId"];
                [dic setObject:vModel.attrName forKey:@"attrName"];
            }
        }
        
        NSDictionary *param = dicList[str];
        model.specPrice = param[@"price"];
        model.specCount = param[@"count"];
        
        [allList addObject:model];
    }

   [normCollectView reloadData];
}

#pragma mark - 拼接规格名字
- (NSString *)getStringAction:(NSArray *)lsit {
    NSString *str = @"";
    for (AttrValueModel *s in lsit) {
        str = [NSString stringWithFormat:@"%@+%@",str,s.attrValueName];
    }
    
    NSMutableString *a = [[NSMutableString  alloc] initWithString:str];
    [a replaceCharactersInRange:NSMakeRange(0, 1) withString:@""];
    return a;
}

#pragma mark - 笛卡儿积算规格组合
- (void)descartes:(NSMutableArray *)dimvalue result:(NSMutableArray *)result layer:(NSInteger)layer curList:(NSMutableArray *)curList{
    
    if (layer < dimvalue.count - 1) {
        if ([dimvalue[layer] count] == 0) {
            [self descartes:dimvalue result:result layer:layer + 1 curList:curList];
        } else {
            for (int i = 0; i < [dimvalue[layer] count]; i++) {
                NSMutableArray *list = [NSMutableArray arrayWithArray:curList];
                [list addObject:dimvalue[layer][i]];
                
                [self descartes:dimvalue result:result layer:layer + 1 curList:list];
                
            }
        }
    } else if (layer == [dimvalue count] - 1) {
        if ([dimvalue[layer] count] == 0) {
            [result addObject:curList];
        } else {
            for (int i = 0; i < [dimvalue[layer] count]; i++) {
                NSMutableArray *list = [NSMutableArray arrayWithArray:curList];
                [list addObject:dimvalue[layer][i]];
                [result addObject:list];
            }
        }
    }
}

#pragma mark - 确认按钮点击事件
- (void)confirmButtonAction  {
    
    if (overallList.count > 0) {
        [self calculateAllListCount];
        
        for (NSInteger i=0; i<space.count; i++) {
            SpecModel *model = allList[i];
            
            NSLog(@"%@---%@", model.specPrice, model.specCount);
        }
    }
    
    [NormEditController destroyShareManager];
    [self dismissViewControllerAnimated:YES completion:nil];
}

#pragma mark - 展开收起点击事件
- (void)unfoldButtonAction:(UIButton *)sender {
    normModel = overallList[sender.tag];
    if (sender.isSelected == NO) {
        normModel.isShow = @"1";
        sender.selected = YES;
    }
    else
    {
        sender.selected = NO;
        normModel.isShow = @"0";
    }
    
    [normCollectView reloadData];
}

@end
